@@ -1,3 +1,4 @@
+//! Types for working with [Anim] data in .nuanmb files.
 use binread::{io::StreamPosition, BinRead};
 use std::{
     convert::{TryFrom, TryInto},
@@ -18,10 +19,11 @@ use thiserror::Error;
 #[cfg(feature = "serde")]
 use serde::{Deserialize, Serialize};
 
-pub use ssbh_lib::{Vector3, Vector4};
+pub use ssbh_lib::{formats::anim::GroupType, Vector3, Vector4};
 
-mod anim_buffer;
-use anim_buffer::*;
+mod buffers;
+use buffers::*;
+mod compression;
 
 use crate::SsbhData;
 
@@ -29,7 +31,7 @@ use crate::SsbhData;
 
 /// The data associated with an [Anim] file.
 /// Supported versions are 2.0 and 2.1.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug)]
 pub struct AnimData {
     pub major_version: u16,
@@ -56,14 +58,12 @@ impl SsbhData for AnimData {
     }
 
     fn write<W: std::io::Write + Seek>(&self, writer: &mut W) -> Result<(), AnimError> {
-        let anim = Anim::try_from(self)?;
-        anim.write(writer)?;
+        Anim::try_from(self)?.write(writer)?;
         Ok(())
     }
 
     fn write_to_file<P: AsRef<Path>>(&self, path: P) -> Result<(), AnimError> {
-        let anim = Anim::try_from(self)?;
-        anim.write_to_file(path)?;
+        Anim::try_from(self)?.write_to_file(path)?;
         Ok(())
     }
 }
@@ -126,6 +126,16 @@ pub enum AnimError {
     )]
     InvalidFinalFrameIndex { final_frame_index: f32 },
 
+    #[error(
+        "Scale options of {:?} cannot be preserved for a {} track.",
+        scale_options,
+        if *compressed {"compressed"} else { "uncompressed"}
+    )]
+    UnsupportedTrackScaleOptions {
+        scale_options: ScaleOptions,
+        compressed: bool,
+    },
+
     /// An error occurred while writing data to a buffer.
     #[error(transparent)]
     Io(#[from] std::io::Error),
@@ -146,7 +156,6 @@ enum AnimVersion {
 
 // TODO: Test this for a small example?
 fn create_anim(data: &AnimData) -> Result<Anim, AnimError> {
-    // TODO: Check that the header matches the version number?
     let version = match (data.major_version, data.minor_version) {
         (2, 0) => Ok(AnimVersion::Version20),
         (2, 1) => Ok(AnimVersion::Version21),
@@ -211,6 +220,7 @@ fn create_anim(data: &AnimData) -> Result<Anim, AnimError> {
         }),
     };
 
+    // TODO: Check that the header matches the version number?
     let anim = Anim {
         major_version: data.major_version,
         minor_version: data.minor_version,
@@ -219,9 +229,9 @@ fn create_anim(data: &AnimData) -> Result<Anim, AnimError> {
     Ok(anim)
 }
 
-fn create_anim_group(g: &GroupData, buffer: &mut Cursor<Vec<u8>>) -> std::io::Result<AnimGroup> {
+fn create_anim_group(g: &GroupData, buffer: &mut Cursor<Vec<u8>>) -> Result<AnimGroup, AnimError> {
     Ok(AnimGroup {
-        group_type: g.group_type.into(),
+        group_type: g.group_type,
         nodes: g
             .nodes
             .iter()
@@ -231,7 +241,7 @@ fn create_anim_group(g: &GroupData, buffer: &mut Cursor<Vec<u8>>) -> std::io::Re
     })
 }
 
-fn create_anim_node(n: &NodeData, buffer: &mut Cursor<Vec<u8>>) -> std::io::Result<AnimNode> {
+fn create_anim_node(n: &NodeData, buffer: &mut Cursor<Vec<u8>>) -> Result<AnimNode, AnimError> {
     Ok(AnimNode {
         name: n.name.as_str().into(), // TODO: Make a convenience method for this?
         tracks: n
@@ -246,17 +256,24 @@ fn create_anim_node(n: &NodeData, buffer: &mut Cursor<Vec<u8>>) -> std::io::Resu
 fn create_anim_track_v2(
     buffer: &mut Cursor<Vec<u8>>,
     t: &TrackData,
-) -> std::io::Result<AnimTrackV2> {
+) -> Result<AnimTrackV2, AnimError> {
     let compression_type = infer_optimal_compression_type(&t.values);
 
-    // Anim tracks are written in the order they appear,
-    // so just use the current position as the data offset.
+    // The current stream position matches the offsets used for Smash Ultimate's anim files.
+    // This assumes we traverse the heirarchy (group -> node -> track) in DFS order.
     let pos_before = buffer.stream_pos()?;
 
     // Pointers for compressed data are relative to the start of the track's data.
-    // This requires using a second writer to correctly calculate offsets.
+    // This requires using a second writer due to how SsbhWrite is implemented.
     let mut track_data = Cursor::new(Vec::new());
-    t.values.write(&mut track_data, compression_type)?;
+
+    // TODO: Add tests for preserving scale compensation?.
+    t.values.write(
+        &mut track_data,
+        compression_type,
+        t.scale_options.inherit_scale,
+        t.scale_options.compensate_scale,
+    )?;
 
     buffer.write_all(&track_data.into_inner())?;
     let pos_after = buffer.stream_pos()?;
@@ -306,7 +323,6 @@ fn read_anim_groups(anim: &Anim) -> Result<Vec<GroupData>, AnimError> {
             major_version: anim.major_version,
             minor_version: anim.minor_version,
         }),
-        // TODO: Rename the ANIM fields to be more consistent (animations -> groups)?
         ssbh_lib::formats::anim::AnimHeader::HeaderV20(header) => {
             read_anim_groups_v20(&header.groups.elements, &header.buffer.elements)
         }
@@ -342,7 +358,7 @@ fn read_anim_groups_v20(
         }
 
         let group = GroupData {
-            group_type: anim_group.group_type.into(),
+            group_type: anim_group.group_type,
             nodes,
         };
         groups.push(group);
@@ -351,6 +367,7 @@ fn read_anim_groups_v20(
     Ok(groups)
 }
 
+// TODO: Add tests for preserving scale inheritance and compensate scale.
 fn create_track_data_v20(
     anim_track: &ssbh_lib::formats::anim::AnimTrackV2,
     anim_buffer: &[u8],
@@ -358,45 +375,19 @@ fn create_track_data_v20(
     let start = anim_track.data_offset as usize;
     let end = start + anim_track.data_size as usize;
     let buffer = &anim_buffer[start..end];
-    let values = read_track_values(buffer, anim_track.flags, anim_track.frame_count as usize)?;
+    let (values, inherit_scale, compensate_scale) =
+        read_track_values(buffer, anim_track.flags, anim_track.frame_count as usize)?;
     Ok(TrackData {
         name: anim_track.name.to_string_lossy(),
         values,
+        scale_options: ScaleOptions {
+            inherit_scale,
+            compensate_scale,
+        },
     })
 }
 
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(Debug, Clone, Copy, PartialEq, Eq)]
-pub enum GroupType {
-    Transform = 1,
-    Visibility = 2,
-    Material = 4,
-    Camera = 5,
-}
-
-impl From<ssbh_lib::formats::anim::GroupType> for GroupType {
-    fn from(t: ssbh_lib::formats::anim::GroupType) -> Self {
-        match t {
-            ssbh_lib::formats::anim::GroupType::Transform => GroupType::Transform,
-            ssbh_lib::formats::anim::GroupType::Visibility => GroupType::Visibility,
-            ssbh_lib::formats::anim::GroupType::Material => GroupType::Material,
-            ssbh_lib::formats::anim::GroupType::Camera => GroupType::Camera,
-        }
-    }
-}
-
-impl From<GroupType> for ssbh_lib::formats::anim::GroupType {
-    fn from(t: GroupType) -> Self {
-        match t {
-            GroupType::Transform => ssbh_lib::formats::anim::GroupType::Transform,
-            GroupType::Visibility => ssbh_lib::formats::anim::GroupType::Visibility,
-            GroupType::Material => ssbh_lib::formats::anim::GroupType::Material,
-            GroupType::Camera => ssbh_lib::formats::anim::GroupType::Camera,
-        }
-    }
-}
-
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug)]
 pub struct GroupData {
     /// The usage type for all the [NodeData] in [nodes](#structfield.nodes)
@@ -404,29 +395,74 @@ pub struct GroupData {
     pub nodes: Vec<NodeData>,
 }
 
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug)]
 pub struct NodeData {
     pub name: String,
     pub tracks: Vec<TrackData>,
 }
 
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+/// The data associated with an [AnimTrackV2].
+///
+/// # Examples
+/// The scale settings can usually be left at the default value.
+/**
+```rust
+use ssbh_data::anim_data::{TrackData, TrackValues, ScaleOptions, Transform};
+
+let track = TrackData {
+    name: "Transform".to_string(),
+    values: TrackValues::Transform(vec![Transform::IDENTITY]),
+    scale_options: ScaleOptions::default()
+};
+```
+ */
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug)]
 pub struct TrackData {
     /// The name of the property to animate.
     pub name: String,
+
+    pub scale_options: ScaleOptions,
+
     /// The frame values for the property specified by [name](#structfield.name).
     ///
     /// Each element in the [TrackValues] provides the value for a single frame.
     /// If the [TrackValues] contains a single element, this track will be considered constant
     /// and repeat that element for each frame in the animation
-    /// until [final_frame_index](struct.AnimData.html#structfield.final_frame_index).
+    /// up to and including [final_frame_index](struct.AnimData.html#structfield.final_frame_index).
     pub values: TrackValues,
 }
 
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
+#[derive(Debug)]
+pub struct ScaleOptions {
+    /// Accumulate the parent's scaling when `true`.
+    ///
+    /// The global scale in world space, is `current_scale * parent_scale` applied recursively on the parent.
+    pub inherit_scale: bool,
+
+    /// Revert the scaling of the immediate parent when `true`.
+    ///
+    /// The final scale relative to the parent is `current_scale * (1 / parent_scale)`.
+    /// For Smash Ultimate, this is not applied recursively on the parent,
+    /// so only the immediate parent's scaling is taken into account.
+    pub compensate_scale: bool,
+}
+
+impl Default for ScaleOptions {
+    fn default() -> Self {
+        // Uncompressed tracks don't allow disabling scale inheritance.
+        // Defaulting to true avoids a potential error.
+        Self {
+            inherit_scale: true,
+            compensate_scale: false,
+        }
+    }
+}
+
 // TODO: Investigate if the names based on the Anim 1.2 property names are accurate.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug, BinRead, PartialEq, SsbhWrite, Default, Clone, Copy)]
 pub struct UvTransform {
     pub scale_u: f32,
@@ -437,9 +473,8 @@ pub struct UvTransform {
 }
 
 /// A decomposed transformation consisting of a scale, rotation, and translation.
-// TODO: Derive default and also add identity transforms?
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(Debug, BinRead, PartialEq, SsbhWrite, Clone, Copy, Default)]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
+#[derive(Debug, PartialEq, Clone, Copy, Default)]
 pub struct Transform {
     /// XYZ scale
     pub scale: Vector3,
@@ -448,14 +483,32 @@ pub struct Transform {
     pub rotation: Vector4,
     /// XYZ translation
     pub translation: Vector3,
-    // TODO: Should this be part of the public API.
-    // TODO: Does this work the same as Maya's scale compensation?
-    // TODO: This should be a bool if it's part of the public API
-    pub compensate_scale: u32,
+}
+
+impl Transform {
+    /// An identity transformation representing no scale, rotation, or translation.
+    pub const IDENTITY: Transform = Transform {
+        scale: Vector3 {
+            x: 1.0,
+            y: 1.0,
+            z: 1.0,
+        },
+        rotation: Vector4 {
+            x: 0.0,
+            y: 0.0,
+            z: 0.0,
+            w: 1.0,
+        },
+        translation: Vector3 {
+            x: 0.0,
+            y: 0.0,
+            z: 0.0,
+        },
+    };
 }
 
 /// A value collection with an element for each frame of the animation.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug, PartialEq)]
 pub enum TrackValues {
     /// Transformations used for camera or skeletal animations.
@@ -509,7 +562,6 @@ impl TrackValues {
         }
     }
 
-    // TODO: Is it worth making this public?
     fn track_type(&self) -> TrackType {
         match self {
             TrackValues::Transform(_) => TrackType::Transform,
@@ -524,6 +576,8 @@ impl TrackValues {
 
 #[cfg(test)]
 mod tests {
+    use crate::assert_hex_eq;
+    use hexlit::hex;
     use ssbh_lib::formats::anim::AnimHeaderV21;
 
     use super::*;
@@ -595,6 +649,7 @@ mod tests {
                     tracks: vec![TrackData {
                         name: String::new(),
                         values: TrackValues::Boolean(vec![true; 4]),
+                        scale_options: ScaleOptions::default(),
                     }],
                 }],
             }],
@@ -665,10 +720,12 @@ mod tests {
                 TrackData {
                     name: "t1".to_string(),
                     values: TrackValues::Float(vec![1.0, 2.0, 3.0]),
+                    scale_options: ScaleOptions::default(),
                 },
                 TrackData {
                     name: "t2".to_string(),
                     values: TrackValues::PatternIndex(vec![4, 5]),
+                    scale_options: ScaleOptions::default(),
                 },
             ],
         };
@@ -886,4 +943,293 @@ mod tests {
             ]))
         );
     }
+
+    #[test]
+    fn read_v20_track_compressed_inherit_scale() {
+        // assist/shovelknight/model/body/c00/model.nuanmb, ArmL, Transform
+        // Scale type set to 2 to test for scale inheritance.
+        let buffer = hex!(
+            // header
+            04000600 a0002b00 cc000000 02000000
+            // scale compression
+            0000803f 0000803f 10000000 00000000
+            0000803f 0000803f 10000000 00000000
+            0000803f 0000803f 10000000 00000000
+            // rotation compression
+            00000000 b9bc433d 0d000000 00000000
+            e27186bd 00000000 0d000000 00000000
+            00000000 ada2273f 10000000 00000000
+            // translation compression
+            16a41d40 16a41d40 10000000 00000000
+            00000000 00000000 10000000 00000000
+            00000000 00000000 10000000 00000000
+            // default value
+            0000803f 0000803f 0000803f
+            00000000 00000000 00000000 0000803f
+            16a41d40 00000000 00000000 00000000 00e0ff03
+            // compressed values
+            00f8ff00 e0ff1f
+        );
+
+        let data = create_track_data_v20(
+            &AnimTrackV2 {
+                name: "abc".into(),
+                flags: TrackFlags {
+                    track_type: TrackType::Transform,
+                    compression_type: CompressionType::Compressed,
+                },
+                frame_count: 2,
+                unk_flags: UnkTrackFlags::new(),
+                data_offset: 0,
+                data_size: buffer.len() as u64,
+            },
+            &buffer,
+        )
+        .unwrap();
+
+        // TODO: This should test the values, but this overlaps with anim_buffer tests?
+        assert_eq!("abc", data.name);
+        assert_eq!(true, data.scale_options.inherit_scale);
+    }
+
+    #[test]
+    fn write_v20_track_compressed_inherit_scale() {
+        // assist/shovelknight/model/body/c00/model.nuanmb, ArmL, Transform
+        // The header scale type should be 2 here instead of 1.
+        let buffer = hex!(
+            // header
+            04000e00 a0002b00 cc000000 02000000
+            // scale compression
+            0000803f 0000803f 10000000 00000000
+            0000803f 0000803f 10000000 00000000
+            0000803f 0000803f 10000000 00000000
+            // rotation compression
+            00000000 b9bc433d 0d000000 00000000
+            e27186bd 00000000 0d000000 00000000
+            00000000 ada2273f 10000000 00000000
+            // translation compression
+            16a41d40 16a41d40 10000000 00000000
+            00000000 00000000 10000000 00000000
+            00000000 00000000 10000000 00000000
+            // default value
+            0000803f 0000803f 0000803f
+            00000000 00000000 00000000 0000803f
+            16a41d40 00000000 00000000 00000000 00e0ff03
+            // compressed values
+            00f8ff00 e0ff1f
+        );
+
+        // Use a large enough frame count to ensure the writer chooses compression.
+        let mut writer = Cursor::new(buffer.to_vec());
+        let track = create_anim_track_v2(
+            &mut writer,
+            &TrackData {
+                name: "abc".into(),
+                values: TrackValues::Transform(vec![Transform::default(); 64]),
+                scale_options: ScaleOptions {
+                    inherit_scale: true,
+                    compensate_scale: false,
+                },
+            },
+        )
+        .unwrap();
+
+        assert_eq!("abc", track.name.to_string_lossy());
+        assert_eq!(64, track.frame_count);
+        // TODO: This overlaps with anim_buffer tests?
+        // Just check the header flags for now.
+        assert_hex_eq!(&writer.get_ref()[..4], &buffer[..4]);
+    }
+
+    #[test]
+    fn read_v20_track_compressed_no_scale_inheritance() {
+        // assist/shovelknight/model/body/c00/model.nuanmb, ArmL, Transform
+        // The header scale type was changed to 1.
+        let buffer = hex!(
+            // header
+            04000500 a0002b00 cc000000 02000000
+            // scale compression
+            0000803f 0000803f 10000000 00000000
+            0000803f 0000803f 10000000 00000000
+            0000803f 0000803f 10000000 00000000
+            // rotation compression
+            00000000 b9bc433d 0d000000 00000000
+            e27186bd 00000000 0d000000 00000000
+            00000000 ada2273f 10000000 00000000
+            // translation compression
+            16a41d40 16a41d40 10000000 00000000
+            00000000 00000000 10000000 00000000
+            00000000 00000000 10000000 00000000
+            // default value
+            0000803f 0000803f 0000803f
+            00000000 00000000 00000000 0000803f
+            16a41d40 00000000 00000000 00000000 00e0ff03
+            // compressed values
+            00f8ff00 e0ff1f
+        );
+
+        let data = create_track_data_v20(
+            &AnimTrackV2 {
+                name: "abc".into(),
+                flags: TrackFlags {
+                    track_type: TrackType::Transform,
+                    compression_type: CompressionType::Compressed,
+                },
+                frame_count: 2,
+                unk_flags: UnkTrackFlags::new(),
+                data_offset: 0,
+                data_size: buffer.len() as u64,
+            },
+            &buffer,
+        )
+        .unwrap();
+
+        // TODO: This should test the values, but this overlaps with anim_buffer tests?
+        assert_eq!("abc", data.name);
+        assert_eq!(false, data.scale_options.inherit_scale);
+    }
+
+    #[test]
+    fn write_v20_track_compressed_no_scale_inheritance() {
+        // assist/shovelknight/model/body/c00/model.nuanmb, ArmL, Transform
+        // The header scale type should be 1 here instead of 2.
+        let buffer = hex!(
+            // header
+            04000d00 a0002b00 cc000000 02000000
+            // scale compression
+            0000803f 0000803f 10000000 00000000
+            0000803f 0000803f 10000000 00000000
+            0000803f 0000803f 10000000 00000000
+            // rotation compression
+            00000000 b9bc433d 0d000000 00000000
+            e27186bd 00000000 0d000000 00000000
+            00000000 ada2273f 10000000 00000000
+            // translation compression
+            16a41d40 16a41d40 10000000 00000000
+            00000000 00000000 10000000 00000000
+            00000000 00000000 10000000 00000000
+            // default value
+            0000803f 0000803f 0000803f
+            00000000 00000000 00000000 0000803f
+            16a41d40 00000000 00000000 00000000 00e0ff03
+            // compressed values
+            00f8ff00 e0ff1f
+        );
+
+        // Use a large enough frame count to ensure the writer chooses compression.
+        let mut writer = Cursor::new(buffer.to_vec());
+        let track = create_anim_track_v2(
+            &mut writer,
+            &TrackData {
+                name: "abc".into(),
+                values: TrackValues::Transform(vec![Transform::default(); 64]),
+                scale_options: ScaleOptions {
+                    inherit_scale: false,
+                    compensate_scale: false,
+                },
+            },
+        )
+        .unwrap();
+
+        assert_eq!("abc", track.name.to_string_lossy());
+        assert_eq!(64, track.frame_count);
+        // Just check the header flags for now.
+        assert_hex_eq!(&writer.get_ref()[..4], &buffer[..4]);
+    }
+
+    #[test]
+    fn read_v20_track_uncompressed() {
+        // assist/shovelknight/model/body/c00/model.nuanmb, FingerL11, Transform
+        let buffer = hex!(
+            0000803f 0000803f 0000803f          // scale
+            00000000 00000000 00000000          // translation
+            0000803f bea4c13f_79906ebe f641bebe // rotation
+            01000000                            // compensate scale
+        );
+
+        let data = create_track_data_v20(
+            &AnimTrackV2 {
+                name: "abc".into(),
+                flags: TrackFlags {
+                    track_type: TrackType::Transform,
+                    compression_type: CompressionType::ConstTransform,
+                },
+                frame_count: 1,
+                unk_flags: UnkTrackFlags::new(),
+                data_offset: 0,
+                data_size: buffer.len() as u64,
+            },
+            &buffer,
+        )
+        .unwrap();
+
+        // TODO: This should test the values, but this overlaps with anim_buffer tests?
+        assert_eq!("abc", data.name);
+        // Uncompressed transforms seem to inherit scale with ConstTransform.
+        // TODO: Investigate if uncompressed transforms always inherit scale.
+        assert_eq!(true, data.scale_options.inherit_scale);
+    }
+
+    #[test]
+    fn write_v20_track_uncompressed_inherit_scale() {
+        // assist/shovelknight/model/body/c00/model.nuanmb, FingerL11, Transform
+        let mut buffer = Cursor::new(
+            hex!(
+                0000803f 0000803f 0000803f          // scale
+                00000000 00000000 00000000          // translation
+                0000803f bea4c13f_79906ebe f641bebe // rotation
+                01000000                            // compensate scale
+            )
+            .to_vec(),
+        );
+        let track = create_anim_track_v2(
+            &mut buffer,
+            &TrackData {
+                name: "abc".into(),
+                values: TrackValues::Transform(vec![Transform {
+                    translation: Vector3::new(1.51284, -0.232973, -0.371597),
+                    rotation: Vector4::new(0.0, 0.0, 0.0, 1.0),
+                    scale: Vector3::new(1.0, 1.0, 1.0),
+                }]),
+                scale_options: ScaleOptions {
+                    inherit_scale: true,
+                    compensate_scale: true,
+                },
+            },
+        )
+        .unwrap();
+
+        assert_eq!("abc", track.name.to_string_lossy());
+        assert_eq!(1, track.frame_count);
+        // TODO: Test additional fields.
+    }
+
+    #[test]
+    fn write_v20_track_uncompressed_no_scale_inheritance() {
+        // Uncompressed tracks always use scale inheritance.
+        // Single frame transform tracks won't be compressed,
+        // so this is not a valid operation.
+        let result = create_anim_track_v2(
+            &mut Cursor::new(Vec::new()),
+            &TrackData {
+                name: "abc".into(),
+                values: TrackValues::Transform(vec![Transform::default()]),
+                scale_options: ScaleOptions {
+                    inherit_scale: false,
+                    compensate_scale: false,
+                },
+            },
+        );
+
+        assert!(matches!(
+            result,
+            Err(AnimError::UnsupportedTrackScaleOptions {
+                scale_options: ScaleOptions {
+                    inherit_scale: false,
+                    compensate_scale: false
+                },
+                compressed: false
+            })
+        ));
+    }
 }
