@@ -1,6 +1,36 @@
+//! # ssbh_data
+//!
+//! ssbh_data provides a more intuitive and minimal API built on ssbh_lib.
+//!
+//! ## Features
+//! The high level nature of ssbh_data makes it easier to integrate with application code than ssbh_lib.
+//! Python bindings are also available with [ssbh_data_py](https://github.com/ScanMountGoat/ssbh_data_py).
+//! - Automatic decoding and encoding of buffers and compressed data
+//! - Usage of standard Rust types like [Vec] and [String]
+//! - Support for converting files to and from supported versions
+//! - Simpler output when serializing and deserializing
+//! - Errors for invalid data such as out of bounds vertex indices
+//! - Modifications are less likely to produce an invalid file due to reduced dependencies between fields
+//!
+//! ## Getting Started
+//!
+//!```no_run
+//!use ssbh_data::SsbhData;
+//!
+//!# fn main() -> Result<(), Box<dyn std::error::Error>> {
+//!let data = ssbh_data::mesh_data::MeshData::from_file("model.numshb")?;
+//!# Ok(())
+//!# }
+//!```
+
+pub mod adj_data;
 pub mod anim_data;
+pub mod matl_data;
 pub mod mesh_data;
+pub mod meshex_data;
 pub mod modl_data;
+#[doc(hidden)]
+pub mod shdr_data;
 pub mod skel_data;
 
 use std::error::Error;
@@ -12,9 +42,10 @@ use binread::io::{Seek, SeekFrom};
 use binread::BinReaderExt;
 use binread::{BinRead, BinResult};
 use half::f16;
+
 use ssbh_lib::SsbhArray;
 
-// TODO: Should this be part of a prelude?
+/// Functions for reading and writing supported formats.
 pub trait SsbhData: Sized {
     type WriteError: Error;
     // TODO: Also specify the read error type?
@@ -115,6 +146,37 @@ fn create_ssbh_array<T, B: BinRead, F: Fn(&T) -> B>(elements: &[T], create_b: F)
     elements.iter().map(create_b).collect::<Vec<B>>().into()
 }
 
+#[cfg(test)]
+pub(crate) fn group_hex(a: &str, words_per_line: usize) -> String {
+    use itertools::Itertools;
+
+    // TODO: Find a cleaner way of doing this.
+    // ex: "FFFFFFFF FFFFFFFF FFFFFFFF FFFFFFFF..."
+    let words = a
+        .chars()
+        .collect::<Vec<char>>()
+        .chunks(8)
+        .map(|c| c.iter().collect::<String>())
+        .collect::<Vec<String>>();
+
+    words.chunks(words_per_line).map(|c| c.join(" ")).join("\n")
+}
+
+#[cfg(test)]
+macro_rules! assert_hex_eq {
+    ($a:expr, $b:expr) => {
+        assert!(
+            $a == $b,
+            "\n{} !=\n{}",
+            crate::group_hex(&hex::encode($a), 8),
+            crate::group_hex(&hex::encode($b), 8)
+        )
+    };
+}
+
+#[cfg(test)]
+pub(crate) use assert_hex_eq;
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -233,9 +295,11 @@ mod tests {
     #[test]
     fn u8_clamped() {
         assert_eq!(0u8, get_u8_clamped(-1.0f32));
-        assert_eq!(0u8, get_u8_clamped(0.0f32));
-        assert_eq!(128u8, get_u8_clamped(128f32 / 255f32));
-        assert_eq!(255u8, get_u8_clamped(1.0f32));
+
+        for u in 0..=255u8 {
+            assert_eq!(u, get_u8_clamped(u as f32 / 255.0f32));
+        }
+
         assert_eq!(255u8, get_u8_clamped(2.0f32));
     }
 }
