@@ -1,6 +1,8 @@
 use std::convert::TryInto;
 
-use ssbh_data::{anim_data::AnimData, mesh_data::MeshData, skel_data::SkelData, SsbhData};
+use ssbh_data::{
+    anim_data::AnimData, matl_data::MatlData, mesh_data::MeshData, skel_data::SkelData, SsbhData,
+};
 use ssbh_lib::SsbhFile;
 
 fn main() {
@@ -34,6 +36,15 @@ fn main() {
             data.write_to_file(&args[2]).unwrap();
             println!("AnimData -> Anim -> File: {:?}", start.elapsed());
         }
+        SsbhFile::Matl(matl) => {
+            let start = std::time::Instant::now();
+            let data: MatlData = matl.try_into().unwrap();
+            println!("Matl -> MatlData: {:?}", start.elapsed());
+
+            let start = std::time::Instant::now();
+            data.write_to_file(&args[2]).unwrap();
+            println!("MatlData -> Matl -> File: {:?}", start.elapsed());
+        }
         _ => (),
     }
 }
