@@ -1,3 +1,4 @@
+//! Types for working with [Mesh] data in .numshb files.
 use binread::{io::Cursor, BinRead};
 use binread::{BinReaderExt, BinResult};
 use half::f16;
@@ -50,12 +51,11 @@ enum AttributeUsage {
     ColorSet,
 }
 
-/// Errors while creating a [Mesh] from [MeshObjectData].
+/// Errors while creating a [Mesh] from [MeshData].
 #[derive(Error, Debug)]
 
 pub enum MeshError {
-    /// The attributes for a [MeshObject] would have different number of elements,
-    /// so the vertex count cannot be determined.
+    /// The attributes have a different number of elements, so the vertex count cannot be determined.
     #[error("Attribute data lengths do not match. Failed to determined vertex count.")]
     AttributeDataLengthMismatch,
 
@@ -72,6 +72,12 @@ pub enum MeshError {
         vertex_count: usize,
     },
 
+    #[error(
+        "Vertex index count {} is not a multiple of 3. Only triangles are supported.",
+        vertex_index_count
+    )]
+    NonTriangulatedFaces { vertex_index_count: usize },
+
     /// Creating a [Mesh] file for the given version is not supported.
     #[error(
         "Creating a version {}.{} mesh is not supported.",
@@ -141,7 +147,7 @@ impl From<AttributeUsageV8> for AttributeUsage {
 }
 
 /// Assigns a weight to a particular vertex.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(BinRead, Debug, Clone)]
 pub struct VertexWeight {
     pub vertex_index: u32,
@@ -344,7 +350,7 @@ fn read_rigging_data(
 }
 
 /// A collection of vertex weights for all the vertices influenced by a bone.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone)]
 pub struct BoneInfluence {
     pub bone_name: String,
@@ -353,7 +359,7 @@ pub struct BoneInfluence {
 
 /// The data associated with a [Mesh] file.
 /// Supported versions are 1.8, 1.9, and 1.10.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone)]
 pub struct MeshData {
     pub major_version: u16,
@@ -409,9 +415,32 @@ impl TryFrom<&Mesh> for MeshData {
 ///
 /// Vertex attribute data is stored in collections of [AttributeData] grouped by usage.
 /// Each [AttributeData] will have its data indexed by [vertex_indices](struct.MeshAttributeV10.html.#structfield.vertex_indices),
-/// so all [data](struct.AttributeData.html#structfield.data) should have the number of elements.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-#[derive(Debug, Clone)]
+/// so all [VectorData] should have the number of elements.
+///
+/// # Examples
+/**
+```rust
+use ssbh_data::mesh_data::{MeshObjectData, AttributeData, VectorData};
+
+let object = MeshObjectData {
+    name: "triangle".to_string(),
+    vertex_indices: vec![0, 1, 2],
+    positions: vec![
+        AttributeData {
+            name: "Position0".into(),
+            data: VectorData::Vector3(vec![
+                [-1.0, 1.0, 0.0],
+                [-1.0, -1.0, 0.0],
+                [1.0, -1.0, 0.0]
+            ])
+        }
+    ],
+    ..MeshObjectData::default()
+};
+```
+ */
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
+#[derive(Debug, Clone, Default)]
 pub struct MeshObjectData {
     /// The name of this object.
     pub name: String,
@@ -419,6 +448,9 @@ pub struct MeshObjectData {
     pub sub_index: u64,
     /// The name of the parent bone. The empty string represents no parent for mesh objects that are not single bound.
     pub parent_bone_name: String,
+    pub sort_bias: i32,
+    pub disable_depth_write: bool,
+    pub disable_depth_test: bool,
     /// Vertex indices for the data for all [AttributeData] for this [MeshObjectData].
     pub vertex_indices: Vec<u32>,
     pub positions: Vec<AttributeData>,
@@ -434,7 +466,7 @@ pub struct MeshObjectData {
 }
 
 /// Data corresponding to a named vertex attribute such as `"Position0"` or `"colorSet1"`.
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone)]
 pub struct AttributeData {
     pub name: String,
@@ -446,7 +478,7 @@ pub struct AttributeData {
 /// The precision when saving is inferred based on supported data types for the version specified in the [MeshData].
 /// For example, position attributes will prefer the highest available precision, and color sets will prefer the lowest available precision.
 /// *The data type selected for saving may change between releases but will always retain the specified component count such as [VectorData::Vector2] vs [VectorData::Vector4].*
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
+#[cfg_attr(feature = "serde1", derive(Serialize, Deserialize))]
 #[derive(Debug, Clone, PartialEq)]
 pub enum VectorData {
     Vector2(Vec<[f32; 2]>),
@@ -586,6 +618,9 @@ fn read_mesh_objects(mesh: &Mesh) -> Result<Vec<MeshObjectData>, Box<dyn Error>>
             texture_coordinates,
             color_sets,
             bone_influences,
+            sort_bias: mesh_object.sort_bias,
+            disable_depth_test: mesh_object.depth_flags.disable_depth_test != 0,
+            disable_depth_write: mesh_object.depth_flags.disable_depth_write != 0,
         };
 
         mesh_objects.push(data);
@@ -618,11 +653,10 @@ fn create_mesh(data: &MeshData) -> Result<Mesh, MeshError> {
     let all_positions: Vec<geometry_tools::glam::Vec3A> = data
         .objects
         .iter()
-        .map(|o| match o.positions.first() {
+        .flat_map(|o| match o.positions.first() {
             Some(attribute) => attribute.data.to_glam_vec3a(),
             None => Vec::new(),
         })
-        .flatten()
         .collect();
 
     let mesh = Mesh {
@@ -760,24 +794,37 @@ fn create_vertex_weights(
     }
 }
 
-fn get_size_in_bytes_v10(data_type: &AttributeDataTypeV10) -> usize {
-    match data_type {
-        AttributeDataTypeV10::Float3 => std::mem::size_of::<f32>() * 3,
-        AttributeDataTypeV10::Byte4 => std::mem::size_of::<u8>() * 4,
-        AttributeDataTypeV10::HalfFloat4 => std::mem::size_of::<f16>() * 4,
-        AttributeDataTypeV10::HalfFloat2 => std::mem::size_of::<f16>() * 2,
-        AttributeDataTypeV10::Float4 => std::mem::size_of::<f32>() * 4,
-        AttributeDataTypeV10::Float2 => std::mem::size_of::<f32>() * 2,
+// TODO: Make these methods.
+trait AttributeDataTypeV10Ext {
+    fn get_size_in_bytes_v10(&self) -> usize;
+}
+
+impl AttributeDataTypeV10Ext for AttributeDataTypeV10 {
+    fn get_size_in_bytes_v10(&self) -> usize {
+        match self {
+            AttributeDataTypeV10::Float3 => std::mem::size_of::<f32>() * 3,
+            AttributeDataTypeV10::Byte4 => std::mem::size_of::<u8>() * 4,
+            AttributeDataTypeV10::HalfFloat4 => std::mem::size_of::<f16>() * 4,
+            AttributeDataTypeV10::HalfFloat2 => std::mem::size_of::<f16>() * 2,
+            AttributeDataTypeV10::Float4 => std::mem::size_of::<f32>() * 4,
+            AttributeDataTypeV10::Float2 => std::mem::size_of::<f32>() * 2,
+        }
     }
 }
 
-fn get_size_in_bytes_v8(data_type: &AttributeDataTypeV8) -> usize {
-    match data_type {
-        AttributeDataTypeV8::Float3 => std::mem::size_of::<f32>() * 3,
-        AttributeDataTypeV8::HalfFloat4 => std::mem::size_of::<f16>() * 4,
-        AttributeDataTypeV8::Float2 => std::mem::size_of::<f32>() * 2,
-        AttributeDataTypeV8::Byte4 => std::mem::size_of::<u8>() * 4,
-        AttributeDataTypeV8::Float4 => std::mem::size_of::<f32>() * 4,
+trait AttributeDataTypeV8Ext {
+    fn get_size_in_bytes_v8(&self) -> usize;
+}
+
+impl AttributeDataTypeV8Ext for AttributeDataTypeV8 {
+    fn get_size_in_bytes_v8(&self) -> usize {
+        match self {
+            AttributeDataTypeV8::Float3 => std::mem::size_of::<f32>() * 3,
+            AttributeDataTypeV8::HalfFloat4 => std::mem::size_of::<f16>() * 4,
+            AttributeDataTypeV8::Float2 => std::mem::size_of::<f32>() * 2,
+            AttributeDataTypeV8::Byte4 => std::mem::size_of::<u8>() * 4,
+            AttributeDataTypeV8::Float4 => std::mem::size_of::<f32>() * 4,
+        }
     }
 }
 
@@ -796,7 +843,7 @@ enum VertexIndices {
 fn create_attributes(
     data: &MeshObjectData,
     version: MeshVersion,
-) -> ([(u32, AttributeBufferData); 4], MeshAttributes) {
+) -> ([(u32, VersionedVectorData); 4], MeshAttributes) {
     match version {
         MeshVersion::Version108 => create_attributes_v8(data),
         MeshVersion::Version109 => create_attributes_v9(data),
@@ -859,6 +906,12 @@ fn create_mesh_object(
     vertex_buffer2_offset: &mut u64,
     index_buffer: &mut Cursor<Vec<u8>>,
 ) -> Result<MeshObject, MeshError> {
+    if data.vertex_indices.len() % 3 != 0 {
+        return Err(MeshError::NonTriangulatedFaces {
+            vertex_index_count: data.vertex_indices.len(),
+        });
+    }
+
     let vertex_count = calculate_vertex_count(data)?;
 
     // Check for out of bounds vertex accesses.
@@ -888,6 +941,7 @@ fn create_mesh_object(
     let vertex_buffer1_offset = buffer1.position();
     let vertex_buffer3_offset = buffer3.position();
 
+    // TODO: This is pretty convoluted.
     let (buffer_info, attributes) = create_attributes(data, version);
 
     let stride0 = buffer_info[0].0;
@@ -936,11 +990,10 @@ fn create_mesh_object(
         } else {
             1
         },
-        sort_bias: 0, // TODO: Preserve sort bias
+        sort_bias: data.sort_bias,
         depth_flags: DepthFlags {
-            // TODO: Preserve depth flags
-            disable_depth_write: 0,
-            disable_depth_test: 0,
+            disable_depth_write: if data.disable_depth_write { 1 } else { 0 },
+            disable_depth_test: if data.disable_depth_test { 1 } else { 0 },
         },
         bounding_info: calculate_bounding_info(&positions),
         attributes,
@@ -997,6 +1050,7 @@ fn calculate_vertex_count(data: &MeshObjectData) -> Result<usize, MeshError> {
             None => Ok(0),
         }
     } else {
+        // TODO: Add the attribute lengths to the error?
         Err(MeshError::AttributeDataLengthMismatch)
     }
 }
@@ -1048,19 +1102,11 @@ fn transform_inner(data: &VectorData, transform: &[[f32; 4]; 4], w: f32) -> Vect
 # use ssbh_data::mesh_data::{VectorData, AttributeData, MeshObjectData, transform_points};
 # let mesh_object_data = MeshObjectData {
 #     name: "abc".into(),
-#     sub_index: 0,
-#     parent_bone_name: "".into(),
-#     vertex_indices: Vec::new(),
 #     positions: vec![AttributeData {
 #         name: "Position0".into(),
 #         data: VectorData::Vector3(Vec::new())
 #     }],
-#     normals: Vec::new(),
-#     binormals: Vec::new(),
-#     tangents: Vec::new(),
-#     texture_coordinates: Vec::new(),
-#     color_sets: Vec::new(),
-#     bone_influences: Vec::new(),
+#     ..MeshObjectData::default()
 # };
 // A scaling matrix for x, y, and z.
 let transform = [
@@ -1086,19 +1132,11 @@ pub fn transform_points(data: &VectorData, transform: &[[f32; 4]; 4]) -> VectorD
 # use ssbh_data::mesh_data::{VectorData, AttributeData, MeshObjectData, transform_vectors};
 # let mesh_object_data = MeshObjectData {
 #     name: "abc".into(),
-#     sub_index: 0,
-#     parent_bone_name: "".into(),
-#     vertex_indices: Vec::new(),
-#     positions: Vec::new(),
 #     normals: vec![AttributeData {
 #         name: "Normal0".into(),
 #         data: VectorData::Vector3(Vec::new())
 #     }],
-#     binormals: Vec::new(),
-#     tangents: Vec::new(),
-#     texture_coordinates: Vec::new(),
-#     color_sets: Vec::new(),
-#     bone_influences: Vec::new(),
+#     ..MeshObjectData::default()
 # };
 // A scaling matrix for x, y, and z.
 let transform = [
@@ -1436,21 +1474,21 @@ mod tests {
 
     #[test]
     fn size_in_bytes_attributes_v10() {
-        assert_eq!(4, get_size_in_bytes_v10(&AttributeDataTypeV10::Byte4));
-        assert_eq!(8, get_size_in_bytes_v10(&AttributeDataTypeV10::Float2));
-        assert_eq!(12, get_size_in_bytes_v10(&AttributeDataTypeV10::Float3));
-        assert_eq!(16, get_size_in_bytes_v10(&AttributeDataTypeV10::Float4));
-        assert_eq!(4, get_size_in_bytes_v10(&AttributeDataTypeV10::HalfFloat2));
-        assert_eq!(8, get_size_in_bytes_v10(&AttributeDataTypeV10::HalfFloat4));
+        assert_eq!(4, AttributeDataTypeV10::Byte4.get_size_in_bytes_v10());
+        assert_eq!(8, AttributeDataTypeV10::Float2.get_size_in_bytes_v10());
+        assert_eq!(12, AttributeDataTypeV10::Float3.get_size_in_bytes_v10());
+        assert_eq!(16, AttributeDataTypeV10::Float4.get_size_in_bytes_v10());
+        assert_eq!(4, AttributeDataTypeV10::HalfFloat2.get_size_in_bytes_v10());
+        assert_eq!(8, AttributeDataTypeV10::HalfFloat4.get_size_in_bytes_v10());
     }
 
     #[test]
     fn size_in_bytes_attributes_v8() {
-        assert_eq!(4, get_size_in_bytes_v8(&AttributeDataTypeV8::Byte4));
-        assert_eq!(8, get_size_in_bytes_v8(&AttributeDataTypeV8::Float2));
-        assert_eq!(12, get_size_in_bytes_v8(&AttributeDataTypeV8::Float3));
-        assert_eq!(16, get_size_in_bytes_v8(&AttributeDataTypeV8::Float4));
-        assert_eq!(8, get_size_in_bytes_v8(&AttributeDataTypeV8::HalfFloat4));
+        assert_eq!(4, AttributeDataTypeV8::Byte4.get_size_in_bytes_v8());
+        assert_eq!(8, AttributeDataTypeV8::Float2.get_size_in_bytes_v8());
+        assert_eq!(12, AttributeDataTypeV8::Float3.get_size_in_bytes_v8());
+        assert_eq!(16, AttributeDataTypeV8::Float4.get_size_in_bytes_v8());
+        assert_eq!(8, AttributeDataTypeV8::HalfFloat4.get_size_in_bytes_v8());
     }
 
     #[test]
@@ -1687,7 +1725,7 @@ mod tests {
             depth_flags: DepthFlags {
                 disable_depth_write: 0,
                 disable_depth_test: 0,
-            }, // TODO: Recreate depth flags.
+            },
             bounding_info: BoundingInfo::default(),
             attributes: MeshAttributes::AttributesV10(Vec::new().into()),
         };
@@ -1763,19 +1801,14 @@ mod tests {
 
     #[test]
     fn create_empty_mesh_object() {
-        create_mesh_object(
+        let object = create_mesh_object(
             &MeshObjectData {
                 name: String::new(),
-                sub_index: 0,
-                parent_bone_name: String::new(),
-                vertex_indices: Vec::new(),
-                positions: Vec::new(),
-                normals: Vec::new(),
-                binormals: Vec::new(),
-                tangents: Vec::new(),
-                texture_coordinates: Vec::new(),
-                color_sets: Vec::new(),
-                bone_influences: Vec::new(),
+                sub_index: 1,
+                sort_bias: -5,
+                disable_depth_test: true,
+                disable_depth_write: false,
+                ..MeshObjectData::default()
             },
             MeshVersion::Version110,
             &mut Cursor::new(Vec::new()),
@@ -1786,6 +1819,11 @@ mod tests {
             &mut Cursor::new(Vec::new()),
         )
         .unwrap();
+
+        assert_eq!(1, object.sub_index);
+        assert_eq!(-5, object.sort_bias);
+        assert_eq!(0, object.depth_flags.disable_depth_write);
+        assert_eq!(1, object.depth_flags.disable_depth_test);
     }
 
     #[test]
@@ -1793,23 +1831,15 @@ mod tests {
         // The vertex count can't be determined since 1 != 2.
         let result = create_mesh_object(
             &MeshObjectData {
-                name: String::new(),
-                sub_index: 0,
-                parent_bone_name: String::new(),
-                vertex_indices: Vec::new(),
                 positions: vec![AttributeData {
                     name: String::new(),
                     data: VectorData::Vector2(vec![[0.0, 0.0], [0.0, 0.0]]),
                 }],
-                normals: Vec::new(),
-                binormals: Vec::new(),
                 tangents: vec![AttributeData {
                     name: String::new(),
                     data: VectorData::Vector2(vec![[0.0, 0.0]]),
                 }],
-                texture_coordinates: Vec::new(),
-                color_sets: Vec::new(),
-                bone_influences: Vec::new(),
+                ..MeshObjectData::default()
             },
             MeshVersion::Version110,
             &mut Cursor::new(Vec::new()),
@@ -1830,23 +1860,16 @@ mod tests {
     fn create_mesh_object_valid_indices() {
         create_mesh_object(
             &MeshObjectData {
-                name: String::new(),
-                sub_index: 0,
-                parent_bone_name: String::new(),
                 vertex_indices: vec![0, 1, 1],
                 positions: vec![AttributeData {
                     name: String::new(),
                     data: VectorData::Vector2(vec![[0.0, 0.0], [0.0, 0.0]]),
                 }],
-                normals: Vec::new(),
-                binormals: Vec::new(),
                 tangents: vec![AttributeData {
                     name: String::new(),
                     data: VectorData::Vector2(vec![[0.0, 0.0], [0.0, 0.0]]),
                 }],
-                texture_coordinates: Vec::new(),
-                color_sets: Vec::new(),
-                bone_influences: Vec::new(),
+                ..MeshObjectData::default()
             },
             MeshVersion::Version110,
             &mut Cursor::new(Vec::new()),
@@ -1859,28 +1882,54 @@ mod tests {
         .unwrap();
     }
 
+    #[test]
+    fn create_mesh_object_quad_faces() {
+        // Currently only triangles are supported.
+        let result = create_mesh_object(
+            &MeshObjectData {
+                vertex_indices: vec![0, 2, 1, 0, 2, 1, 0, 0],
+                positions: vec![AttributeData {
+                    name: String::new(),
+                    data: VectorData::Vector2(vec![[0.0, 0.0], [0.0, 0.0]]),
+                }],
+                tangents: vec![AttributeData {
+                    name: String::new(),
+                    data: VectorData::Vector2(vec![[0.0, 0.0], [0.0, 0.0]]),
+                }],
+                ..MeshObjectData::default()
+            },
+            MeshVersion::Version110,
+            &mut Cursor::new(Vec::new()),
+            &mut Cursor::new(Vec::new()),
+            &mut Cursor::new(Vec::new()),
+            &mut Cursor::new(Vec::new()),
+            &mut 0,
+            &mut Cursor::new(Vec::new()),
+        );
+
+        assert!(matches!(
+            result,
+            Err(MeshError::NonTriangulatedFaces {
+                vertex_index_count: 8,
+            })
+        ));
+    }
+
     #[test]
     fn create_mesh_object_invalid_indices() {
         // Index 2 is out of bounds for the vertex attribute arrays.
         let result = create_mesh_object(
             &MeshObjectData {
-                name: String::new(),
-                sub_index: 0,
-                parent_bone_name: String::new(),
                 vertex_indices: vec![0, 2, 1],
                 positions: vec![AttributeData {
                     name: String::new(),
                     data: VectorData::Vector2(vec![[0.0, 0.0], [0.0, 0.0]]),
                 }],
-                normals: Vec::new(),
-                binormals: Vec::new(),
                 tangents: vec![AttributeData {
                     name: String::new(),
                     data: VectorData::Vector2(vec![[0.0, 0.0], [0.0, 0.0]]),
                 }],
-                texture_coordinates: Vec::new(),
-                color_sets: Vec::new(),
-                bone_influences: Vec::new(),
+                ..MeshObjectData::default()
             },
             MeshVersion::Version110,
             &mut Cursor::new(Vec::new()),
@@ -1905,17 +1954,8 @@ mod tests {
         // Index 0 is out of bounds for the vertex attribute arrays.
         let result = create_mesh_object(
             &MeshObjectData {
-                name: String::new(),
-                sub_index: 0,
-                parent_bone_name: String::new(),
                 vertex_indices: vec![0, 0, 0],
-                positions: Vec::new(),
-                normals: Vec::new(),
-                binormals: Vec::new(),
-                tangents: Vec::new(),
-                texture_coordinates: Vec::new(),
-                color_sets: Vec::new(),
-                bone_influences: Vec::new(),
+                ..MeshObjectData::default()
             },
             MeshVersion::Version110,
             &mut Cursor::new(Vec::new()),
@@ -1934,4 +1974,6 @@ mod tests {
             })
         ));
     }
+
+    // TODO: Test creating a MeshObjectData
 }
